/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include "interface.h"
#include "support.h"
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <stdlib.h>
#include <pthread.h>

struct pr_args {
	char *command;
	char *parm;
};

int swrite (char *str);
void restoreline ();
void listen ();
void *processcommand (void *myargs);
char *substring(int start, int length, char *str);
int stringeq(char *str1, char *str2);
int find_combo_element (GtkTreeModel* list_store, char *param);

/* baudrate settings are defined in <asm/termbits.h>, which is
included by <termios.h> */
#define BAUDRATE B9600            
/* change this definition for the correct port */
#define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define FALSE 0
#define TRUE 1

int fd,gap=1,response=1,busy=0;
struct termios oldtio,newtio;
GtkWidget *window1;
pthread_t listen_thread;


int
main (int argc, char *argv[])
{
	g_thread_init (NULL);
	gdk_threads_init ();

//gdk_threads_enter ();

/* 
Open modem device for reading and writing and not as controlling tty
because we don't want to get killed if linenoise sends CTRL-C.
*/
fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY | O_NDELAY); 
if (fd <0) {perror(MODEMDEVICE); exit(-1); }

tcgetattr(fd,&oldtio); /* save current serial port settings */
bzero(&newtio, sizeof(newtio)); /* clear struct for new port settings */

/* 
BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
CRTSCTS : output hardware flow control (only used if the cable has
all necessary lines. See sect. 7 of Serial-HOWTO)
CS8     : 8n1 (8bit,no parity,1 stopbit)
CLOCAL  : local connection, no modem contol
CREAD   : enable receiving characters
*/

newtio.c_cflag |= (BAUDRATE | CS8 | CLOCAL | CREAD);
         
/*
IGNPAR  : ignore bytes with parity errors
ICRNL   : map CR to NL (otherwise a CR input on the other computer
will not terminate input)
otherwise make device raw (no other input processing)
*/
newtio.c_iflag = IGNPAR | ICRNL;
         
/*
Raw output.
*/
newtio.c_oflag = 0;
         
/*
ICANON  : enable canonical input
disable all echo functionality, and don't send signals to calling program
*/
newtio.c_lflag = ICANON;
         

/* 
now clean the modem line and activate the settings for the port
*/

tcflush(fd, TCIFLUSH);
tcsetattr(fd,TCSANOW,&newtio);


pthread_create( &listen_thread, NULL, listen, NULL);
//gdk_threads_leave ();


#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

gtk_set_locale ();
gtk_init (&argc, &argv);

add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
add_pixmap_directory ("/mnt/media/doc/prog/glade/state0/src");

/*
* The following code was added by Glade to create one of each component
* (except popup menus), just so that you see something after building
* the project. Delete any components that you don't want shown initially.
*/

window1 = create_window1 ();
gtk_widget_show (window1);

gtk_main ();

return 0;
}






int swrite (char *str) {
	if (busy ==1) {
		printf("KILLED, SORRY I'M  BUSY\n");return 0;
	}
	busy =1;
	char *temp= malloc(20);int res; int i=0;

	strcpy(temp, str); strcat(temp, "\r");
	
	while (response == 0 && i < 11) { usleep(10000); i++;}
	while (gap == 0) {usleep(10000); printf(".");}
	response = 0;
	
	res = write(fd, temp, strlen(temp));
	if (res < 0) fputs("write failed!\n", stderr); else printf("OUT: %s\n", str);
	busy=0;

}
	


void listen () {
	char buf[255];int res = 0; int i = 0;
	pthread_t thread;
	struct pr_args *myargs;
	
	while (TRUE){
		res = read(fd,buf,255);
		buf[res]=0;             /* set end of string, so we can printf */
		if (res > 0) {
			printf("IN: %s\n", buf);
			myargs = (struct pr_args *) malloc(sizeof(struct pr_args));
			myargs->command = substring(0,2,buf);
			myargs->parm = substring(2,strlen(buf)-3, buf);
//			gdk_threads_enter ();
			pthread_create( &thread, NULL, processcommand, (void *) myargs);
//			gdk_threads_leave ();
			i = 0; gap = 0;response=1;
			}
		else {
			usleep(10000);
			if (i < 11) i++;
			if (i == 10) {
				printf("GAP\n\n", buf);
				gap = 1;
			}
		}
	}

}




char *substring(int start, int length, char *str){
	int i;
	char *temp;
	temp = (char *) calloc(length, sizeof(char));

	for (i=start; i < length + start; i++)
		{
			temp[i-start] = str[i];
		}
	return temp;
}	

int stringeq(char *str1, char *str2){
	if (strlen(str1) != strlen(str2)) return 0;
	int i;
	
	for (i=0; i < strlen(str1); i++)
		if (str1[i] != str2[i]) return 0;
	return 1;
}

int find_combo_element (GtkTreeModel* list_store, char *param){
	enum
		{
			STRING_COLUMN,
			N_COLUMNS
				};
	GtkTreeIter iter;
	gboolean valid;
	int i = 0;
	
	/* Get the first iter in the list */
	valid = gtk_tree_model_get_iter_first (list_store, &iter);

	while (valid){
		gchar *str_data;
		gtk_tree_model_get (list_store, &iter, STRING_COLUMN, &str_data,-1);
		if (stringeq(str_data,param)) return i;
		i ++;
		valid = gtk_tree_model_iter_next (list_store, &iter);
		}
	return 0;
}


void restoreline () {
	tcsetattr(fd,TCSANOW,&oldtio);
	close(fd);
}


void *processcommand (void *myargs){
	gdk_threads_enter ();
	struct pr_args t = *(struct pr_args *) myargs;
	
	char *str,*param;

	
	str=t.command;
	param=t.parm;

	if (stringeq(str,"MU")) {
		GtkWidget *mute = lookup_widget(GTK_WIDGET(window1), "mute");
		if (stringeq(param,"ON"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mute), 1);
		else if (stringeq(param,"OFF"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mute), 0);
	}
	if (stringeq(str,"PW")) {
		GtkWidget *power = lookup_widget(GTK_WIDGET(window1), "power");
		if (stringeq(param,"ON"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(power), 1);
		else if (stringeq(param,"STANDBY"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(power), 0);
	}
	if (stringeq(str,"MV")) {
		int db = atoi(param) - 80;
		char *temp2 = malloc(6);
		sprintf( temp2, "%i", db);
		
		GtkWidget *volume = lookup_widget(GTK_WIDGET(window1), "volume");
		GtkWidget *voldb = lookup_widget(GTK_WIDGET(window1), "voldb");

		char *temp= malloc(50);
		strcpy(temp,"<span size=\"xx-large\">");
		strcat(temp, temp2);
		strcat(temp, "</span>");
		gtk_label_set_markup (GTK_LABEL(voldb), temp);
		gtk_range_set_value (GTK_RANGE(volume), atof(param));
	}

	if (stringeq(str,"SI")) {
		GtkWidget *source = lookup_widget(GTK_WIDGET(window1), "source");
		gtk_combo_box_set_active(GTK_COMBO_BOX(source),
				find_combo_element(gtk_combo_box_get_model(GTK_COMBO_BOX(source)), param));
	}

	if (stringeq(str,"SR")) {

		GtkWidget *record_source = lookup_widget(GTK_WIDGET(window1), "record_source");
		gtk_combo_box_set_active(GTK_COMBO_BOX(record_source),
				find_combo_element(gtk_combo_box_get_model(GTK_COMBO_BOX(record_source)), param));
	}
	if (stringeq(str,"MS")) {

		GtkWidget *surround_mode = lookup_widget(GTK_WIDGET(window1), "surround_mode");
		gtk_combo_box_set_active(GTK_COMBO_BOX(surround_mode),
				find_combo_element(gtk_combo_box_get_model(GTK_COMBO_BOX(surround_mode)), param));
	}
	if (stringeq(str,"TM")) {

		GtkWidget *tauto = lookup_widget(GTK_WIDGET(window1), "tauto");
		GtkWidget *tmanual = lookup_widget(GTK_WIDGET(window1), "tmanual");
		GtkWidget *fm = lookup_widget(GTK_WIDGET(window1), "fm");
		GtkWidget *am = lookup_widget(GTK_WIDGET(window1), "am");
		if (stringeq(param,"AUTO"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(tauto), 1);
		else if (stringeq(param,"MANUAL"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(tmanual), 1);
		else if (stringeq(param,"FM"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(fm), 1);
		else if (stringeq(param,"AM"))
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(am), 1);
		
	}
	
	if (stringeq(str,"TP")) {

		char *temp= malloc(50);
		strcpy(temp,"<span size=\"xx-large\">");
		strcat(temp, param);
		strcat(temp, "</span>");
		GtkWidget *tpreset = lookup_widget(GTK_WIDGET(window1), "tpreset");
		gtk_label_set_markup (GTK_LABEL(tpreset), temp);

	}
	
	if (stringeq(str,"TF")) {

		GtkWidget *freq = lookup_widget(GTK_WIDGET(window1), "freq");
		char temp[7];
		sprintf(temp,"%s.%s",substring(0,4,param),substring(4,2,param));
		char *temp2= malloc(50);
		strcpy(temp2,"<span size=\"large\"><b>");
		strcat(temp2, temp);
		strcat(temp2, "</b></span>");
		gtk_label_set_markup (GTK_LABEL(freq), temp2);
	}

	if (stringeq(str,"CV")) {
		GtkWidget *window2 = gtk_object_get_data (GTK_OBJECT (window1), "window2");
		if (window2 != NULL){
		
		char *temp = malloc(15);
			
		if (stringeq(substring(0,2,param), "FL")){
			GtkWidget *fl = lookup_widget(GTK_WIDGET(window2), "fl");
			gtk_range_set_value (GTK_RANGE(fl), atof(substring(3,strlen(param)-3, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "fldb");
			sprintf( temp, "<b> %i </b>", atoi(substring(3,strlen(param)-3, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);
			

		}
		if (stringeq(substring(0,2,param), "FR")){
			GtkWidget *fr = lookup_widget(GTK_WIDGET(window2), "fr");
			gtk_range_set_value (GTK_RANGE(fr), atof(substring(3,strlen(param)-3, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "frdb");
			sprintf( temp, "<b> %i </b>", atoi(substring(3,strlen(param)-3, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);
		
		}
		if (stringeq(substring(0,2,param), "SL")){
			GtkWidget *sl = lookup_widget(GTK_WIDGET(window2), "sl");
			gtk_range_set_value (GTK_RANGE(sl), atof(substring(3,strlen(param)-3, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "sldb");
			sprintf( temp, "<b> %i </b>", atoi(substring(3,strlen(param)-3, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);
		
		}
		if (stringeq(substring(0,2,param), "SR")){
			GtkWidget *sr = lookup_widget(GTK_WIDGET(window2), "sr");
			gtk_range_set_value (GTK_RANGE(sr), atof(substring(3,strlen(param)-3, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "srdb");
			sprintf( temp, "<b> %i </b>", atoi(substring(3,strlen(param)-3, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);
		
		}
		if (stringeq(substring(0,2,param), "SB")){
			GtkWidget *sb = lookup_widget(GTK_WIDGET(window2), "sb");
			gtk_range_set_value (GTK_RANGE(sb), atof(substring(3,strlen(param)-3, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "sbdb");
			sprintf( temp, "<b> %i </b>", atoi(substring(3,strlen(param)-3, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);

		}
		if (stringeq(substring(0,2,param), "SW")){
			GtkWidget *sw = lookup_widget(GTK_WIDGET(window2), "sw");
			gtk_range_set_value (GTK_RANGE(sw), atof(substring(3,strlen(param)-3, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "swdb");
			sprintf( temp, "<b> %i </b>", atoi(substring(3,strlen(param)-3, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);
		
		}
		if (stringeq(substring(0,3,param), "SBL")){
			GtkWidget *sbl = lookup_widget(GTK_WIDGET(window2), "sbl");
			gtk_range_set_value (GTK_RANGE(sbl), atof(substring(4,strlen(param)-4, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "sbldb");
			sprintf( temp, "<b> %i </b>", atoi(substring(4,strlen(param)-4, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);

		}
		if (stringeq(substring(0,3,param), "SBR")){
			GtkWidget *sbr = lookup_widget(GTK_WIDGET(window2), "sbr");
			gtk_range_set_value (GTK_RANGE(sbr), atof(substring(4,strlen(param)-4, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "sbrdb");
			sprintf( temp, "<b> %i </b>", atoi(substring(4,strlen(param)-4, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);

		}
		if (stringeq(substring(0,1,param), "C")){
			GtkWidget *c = lookup_widget(GTK_WIDGET(window2), "c");
			gtk_range_set_value (GTK_RANGE(c), atof(substring(2,strlen(param)-2, param)));

			GtkWidget *voldb = lookup_widget(GTK_WIDGET(window2), "cdb");
			sprintf( temp, "<b> %i </b>", atoi(substring(2,strlen(param)-2, param)) );
			gtk_label_set_markup (GTK_LABEL(voldb), temp);
		
		}
		
		gtk_widget_queue_draw(window2);
		free(temp);

		}
		
	}

	free(myargs);
	gtk_widget_queue_draw(window1);
	gdk_window_process_all_updates();
	gdk_threads_leave ();
}
